import { ApolloLink, FetchResult } from 'apollo-link';
import { DocumentNode } from 'graphql';
import { QueryScheduler } from '../scheduler/scheduler';
import { Observer, Subscription, Observable } from '../util/Observable';
import { DataStore } from '../data/store';
import { MutationStore } from '../data/mutations';
import { QueryStore } from '../data/queries';
import { WatchQueryOptions, SubscriptionOptions, MutationOptions } from './watchQueryOptions';
import { ObservableQuery } from './ObservableQuery';
import { QueryListener, ApolloQueryResult, FetchType } from './types';
export interface QueryInfo {
    listeners: QueryListener[];
    invalidated: boolean;
    newData: ApolloQueryResult<any> | null;
    document: DocumentNode | null;
    lastRequestId: number | null;
    observableQuery: ObservableQuery<any> | null;
    subscriptions: Subscription[];
    cancel?: (() => void);
}
export interface QueryPromise {
    promise: Promise<ApolloQueryResult<any>>;
    resolve: (result: ApolloQueryResult<any>) => void;
    reject: (error: Error) => void;
}
export declare class QueryManager<TStore> {
    scheduler: QueryScheduler<TStore>;
    link: ApolloLink;
    mutationStore: MutationStore;
    queryStore: QueryStore;
    dataStore: DataStore<TStore>;
    private deduplicator;
    private queryDeduplication;
    private onBroadcast;
    private idCounter;
    private queries;
    private fetchQueryPromises;
    private queryIdsByName;
    constructor({link, queryDeduplication, store, onBroadcast, ssrMode}: {
        link: ApolloLink;
        queryDeduplication?: boolean;
        store: DataStore<TStore>;
        onBroadcast?: () => void;
        ssrMode?: boolean;
    });
    mutate<T>({mutation, variables, optimisticResponse, updateQueries: updateQueriesByName, refetchQueries, update: updateWithProxyFn, errorPolicy}: MutationOptions): Promise<FetchResult<T>>;
    fetchQuery<T>(queryId: string, options: WatchQueryOptions, fetchType?: FetchType, fetchMoreForQueryId?: string): Promise<FetchResult<T>>;
    queryListenerForObserver<T>(queryId: string, options: WatchQueryOptions, observer: Observer<ApolloQueryResult<T>>): QueryListener;
    watchQuery<T>(options: WatchQueryOptions, shouldSubscribe?: boolean): ObservableQuery<T>;
    query<T>(options: WatchQueryOptions): Promise<ApolloQueryResult<T>>;
    generateQueryId(): string;
    stopQueryInStore(queryId: string): void;
    addQueryListener(queryId: string, listener: QueryListener): void;
    updateQueryWatch(queryId: string, document: DocumentNode, options: WatchQueryOptions): () => void;
    addFetchQueryPromise<T>(requestId: number, promise: Promise<ApolloQueryResult<T>>, resolve: (result: ApolloQueryResult<T>) => void, reject: (error: Error) => void): void;
    removeFetchQueryPromise(requestId: number): void;
    addObservableQuery<T>(queryId: string, observableQuery: ObservableQuery<T>): void;
    removeObservableQuery(queryId: string): void;
    resetStore(): Promise<ApolloQueryResult<any>[]>;
    startQuery<T>(queryId: string, options: WatchQueryOptions, listener: QueryListener): string;
    startGraphQLSubscription(options: SubscriptionOptions): Observable<any>;
    stopQuery(queryId: string): void;
    removeQuery(queryId: string): void;
    getCurrentQueryResult<T>(observableQuery: ObservableQuery<T>, sameVariables?: boolean): any;
    getQueryWithPreviousResult<T>(queryIdOrObservable: string | ObservableQuery<T>): {
        previousResult: any;
        variables: {
            [key: string]: any;
        } | undefined;
        document: DocumentNode;
    };
    broadcastQueries(): void;
    private fetchRequest<T>({requestId, queryId, document, options, fetchMoreForQueryId});
    private refetchQueryByName(queryName);
    private generateRequestId();
    private getQuery(queryId);
    private setQuery(queryId, updater);
    private invalidate(invalidated, queryId?, fetchMoreForQueryId?);
}
